@@ -47,6 +47,7 @@ static void unhandled_private_notify (gpointer value);
 static void log_hooks_private_notify (gpointer value);
 
 static gboolean initialized = FALSE;
+static GLogLevelFlags fatal_flags = 0;
 
 static gpointer dead_engine = NULL;
 #define DEAD_ENGINE ((gpointer) &dead_engine)
@@ -155,6 +156,15 @@ log_handler (const gchar    *log_domain,
 
   /* Use the default log handler directly to avoid recurse complaints */
   g_log_default_handler (log_domain, log_level, message, user_data);
+
+  /* Support for the standard G_DEBUG flags */
+  if (((log_level & G_LOG_LEVEL_WARNING) != 0 &&
+       (fatal_flags & G_LOG_LEVEL_WARNING) != 0) ||
+      ((log_level & G_LOG_LEVEL_CRITICAL) != 0 &&
+       (fatal_flags & G_LOG_LEVEL_CRITICAL) != 0))
+    {
+      G_BREAKPOINT ();
+    }
 }
 
 void
@@ -179,6 +189,10 @@ void
 testing_util_init (void)
 {
   GError *error = NULL;
+  const GDebugKey glib_debug_keys[] = {
+    { "fatal-criticals", G_LOG_LEVEL_CRITICAL },
+    { "fatal-warnings",  G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING }
+  };
 
   if (initialized)
     return;
@@ -188,6 +202,13 @@ testing_util_init (void)
 
   g_log_set_default_handler (log_handler, NULL);
 
+  /* Force a breakpoint when the standard GLib debug flags
+   * are used. This is not supplied automatically because
+   * GLib's test utilities change the default.
+   */
+  fatal_flags = g_parse_debug_string (g_getenv ("G_DEBUG"), glib_debug_keys,
+                                      G_N_ELEMENTS (glib_debug_keys));
+
   g_irepository_require_private (g_irepository_get_default (),
                                  BUILDDIR "/libpeas",
                                  "Peas", "1.0", 0, &error);
@@ -228,8 +249,20 @@ testing_util_engine_new_full (gboolean nonglobal_loaders)
                      (GWeakNotify) engine_weak_notify,
                      NULL);
 
-  peas_engine_add_search_path (engine, BUILDDIR "/tests/plugins",
-                                       SRCDIR   "/tests/plugins");
+  /* The plugins that two-deps depends on must be added
+   * to the engine before it. This is used to verify that
+   * the engine will order the plugin list correctly.
+   */
+  peas_engine_add_search_path (engine,
+                               BUILDDIR "/tests/plugins/builtin",
+                               SRCDIR   "/tests/plugins/builtin");
+  peas_engine_add_search_path (engine,
+                               BUILDDIR "/tests/plugins/loadable",
+                               SRCDIR   "/tests/plugins/loadable");
+
+  peas_engine_add_search_path (engine,
+                               BUILDDIR "/tests/plugins",
+                               SRCDIR   "/tests/plugins");
 
   return engine;
 }
@@ -261,9 +294,12 @@ testing_util_run_tests (void)
 
   retval = g_test_run ();
 
-  /* Cannot call this with atexit() because
-   * gcov does not register that it was called.
+  /* Cleanup various data early otherwise some
+   * tools, like gcov, will not process it correctly
    */
+  g_private_replace (&engine_key, NULL);
+  g_private_replace (&unhandled_key, NULL);
+  g_private_replace (&log_hooks_key, NULL);
   peas_engine_shutdown ();
 
   return retval;
@@ -338,20 +374,22 @@ testing_util_pop_log_hooks (void)
   msg = g_string_new ("");
 
   if (unhit_hooks->len != 0)
-    g_string_append (msg, "Log hooks were not triggered:");
+    {
+      g_string_append (msg, "Log hooks were not triggered:");
 
-  if (unhit_hooks->len == 1)
-    {
-      hook = g_ptr_array_index (unhit_hooks, 0);
-      g_string_append_printf (msg, " '%s'", hook->pattern);
-    }
-  else if (unhit_hooks->len > 1)
-    {
-      for (i = 0; i < unhit_hooks->len; ++i)
+      if (unhit_hooks->len == 1)
         {
-          hook = g_ptr_array_index (unhit_hooks, i);
+          hook = g_ptr_array_index (unhit_hooks, 0);
+          g_string_append_printf (msg, " '%s'", hook->pattern);
+        }
+      else
+        {
+          for (i = 0; i < unhit_hooks->len; ++i)
+            {
+              hook = g_ptr_array_index (unhit_hooks, i);
 
-          g_string_append_printf (msg, "\n\t'%s'", hook->pattern);
+              g_string_append_printf (msg, "\n\t'%s'", hook->pattern);
+            }
         }
     }
 
